@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 #   Helper.py keeps the rest of the tools clean - part of SIPVicious tools
-#   Copyright (C) 2007-2020  Sandro Gauci <sandro@enablesecurity.com>
+#   Copyright (C) 2007-2021 Sandro Gauci <sandro@enablesecurity.com>
 #
 #   This program is free software: you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -19,13 +19,12 @@
 
 
 __author__ = "Sandro Gauci <sandro@enablesecurity.com>"
-__version__ = '0.3.0'
+__version__ = '0.3.2'
 
 
 import re
 import sys
 import uuid
-import base64
 import os
 import dbm
 import socket
@@ -33,19 +32,17 @@ import random
 import struct
 import shutil
 import logging
-import optparse
-from urllib.parse import quote
 from random import getrandbits
 from urllib.request import urlopen
 from urllib.error import URLError
 from urllib.parse import urlencode
-from binascii import b2a_hex, a2b_hex
 from binascii import Error as b2aerr
 from .pptable import to_string
+from binascii import b2a_hex, a2b_hex, hexlify
 
-if sys.hexversion < 0x03050000:
+if sys.hexversion < 0x03060000:
     sys.stderr.write(
-        "Please update to python 3.5 or greater to run Sipvicious\r\n")
+        "Please update to python 3.6 or greater to run SIPVicious\r\n")
     sys.exit(1)
 
 
@@ -54,13 +51,11 @@ def standardoptions(parser):
                       help="Increase verbosity")
     parser.add_option('-q', '--quiet', dest="quiet", action="store_true",
                       default=False, help="Quiet mode")
-    parser.add_option("-p", "--port", dest="port", default="5060",
-                      help="Destination port or port ranges of the SIP device - eg -p5060,5061,8000-8100", metavar="PORT")
     parser.add_option("-P", "--localport", dest="localport", default=5060, type="int",
                       help="Source port for our packets", metavar="PORT")
     parser.add_option("-x", "--externalip", dest="externalip",
                       help="IP Address to use as the external ip. Specify this if you have multiple interfaces or if you are behind NAT", metavar="IP")
-    parser.add_option("-b", "--bindingip", dest="bindingip", default='0.0.0.0',
+    parser.add_option("-b", "--bindingip", dest="bindingip", default='',
                       help="By default we bind to all interfaces. This option overrides that and binds to the specified ip address")
     parser.add_option("-t", "--timeout", dest="selecttime", type="float", default=0.005,
                       help="This option allows you to trottle the speed at which packets are sent. Change this if you're losing packets. For example try 0.5.",
@@ -445,7 +440,7 @@ def challengeResponse(auth, method, uri):
     opaque = auth["opaque"]
     algorithm = auth["algorithm"]
     cnonce = ""
-    qop = None  
+    qop = None
     if auth["qop"] != None:
         qop = auth["qop"].split(',')[0]
     result = 'Digest username="%s",realm="%s",nonce="%s",uri="%s"' % (
@@ -526,7 +521,7 @@ def makeRequest(method, fromaddr, toaddr, dsthost, port, callid, srchost='', bra
         headers['t'] = toaddr
         headers['f'] = fromaddr
         if localtag is not None:
-            headers['f'] += ';tag=%s' % localtag.decode('utf-8')
+            headers['f'] += ';tag=%s' % localtag.decode('utf-8', 'ignore')
         headers['i'] = callid
         # if contact is not None:
         headers['m'] = contact
@@ -538,7 +533,7 @@ def makeRequest(method, fromaddr, toaddr, dsthost, port, callid, srchost='', bra
         headers['From'] = fromaddr
         headers['User-Agent'] = useragent
         if localtag is not None:
-            headers['From'] += ';tag=%s' % localtag.decode('utf-8')
+            headers['From'] += ';tag=%s' % localtag.decode('utf-8', 'ignore')
         headers['Call-ID'] = callid
         # if contact is not None:
         headers['Contact'] = contact
@@ -631,6 +626,8 @@ def scanrandom(ipranges, portranges, methods, resume=None, randomstore='.sipvici
         pass
     ipsleft = 0
     for iprange in ipranges:
+        if iprange is None:
+            continue
         startip, endip = iprange
         ipsleft += endip - startip + 1
         hit = 0
@@ -686,6 +683,11 @@ def numToDottedQuad(n):
     return socket.inet_ntoa(struct.pack('!L', n))
 
 
+def colonHexToNum(ip):
+    "convert ipv6 address to long integer"
+    return int(hexlify(socket.inet_pton(socket.AF_INET6, ip)), 16)
+
+
 def ip4range(*args):
     for arg in args:
         r = getranges(arg)
@@ -698,6 +700,12 @@ def ip4range(*args):
             curip += 1
 
 
+def ip6range(*args):
+    for arg in args:
+        if check_ipv6(arg):
+            yield(arg)
+
+
 def getranges(ipstring):
     log = logging.getLogger('getranges')
     if re.match(
@@ -724,7 +732,7 @@ def getranges(ipstring):
             naddr1 = dottedQuadToNum(socket.gethostbyname(ipstring))
             naddr2 = naddr1
         except socket.error:
-            log.info('Could not resolve %s' % ipstring)
+            log.error('Could not resolve %s' % ipstring)
             return
     return((naddr1, naddr2))
 
@@ -784,6 +792,8 @@ def resumeFromIP(ip, args):
     rargs = list()
     nip = dottedQuadToNum(ip)
     for arg in args:
+        if arg is None:
+            continue
         startip, endip = getranges(arg)
         if not foundit:
             if startip <= nip and endip >= nip:
@@ -1141,6 +1151,10 @@ def getAuthHeader(pkt):
 
 
 def check_ipv6(n):
+    log = logging.getLogger('check_ipv6')
+    if '/' in n:
+        log.error('CIDR notation not supported for IPv6 addresses.')
+        return False
     try:
         socket.inet_pton(socket.AF_INET6, n)
         return True
