@@ -1,10 +1,9 @@
-#!/usr/bin/env python3
 # SIPvicious password cracker - svcrack
 
 __GPL__ = """
 
    SIPvicious password cracker is an online password guessing tool for SIP devices
-   Copyright (C) 2007-2020  Sandro Gauci <sandro@enablesecurity.com>
+   Copyright (C) 2007-2021 Sandro Gauci <sandro@enablesecurity.com>
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -20,7 +19,6 @@ __GPL__ = """
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """
 
-import base64
 import logging
 import dbm
 import random
@@ -33,10 +31,10 @@ import traceback
 from sys import exit
 from datetime import datetime
 from optparse import OptionParser
-from .libs.pptable import to_string
-from .libs.svhelper import ( __version__, mysendto, reportBugToAuthor,
+from sipvicious.libs.pptable import to_string
+from sipvicious.libs.svhelper import ( __version__, mysendto, reportBugToAuthor,
     numericbrute, dictionaryattack, packetcounter, check_ipv6,
-    createTag, makeRequest, getAuthHeader, getNonce, getOpaque, 
+    createTag, makeRequest, getAuthHeader, getNonce, getOpaque,
     getAlgorithm, getQop, getCID, getRealm, getCredentials, getRange,
     standardscanneroptions, standardoptions, calcloglevel, resumeFrom
 )
@@ -82,7 +80,12 @@ class ASipOfRedWine:
         self.challenges = list()
         self.crackmode = crackmode
         self.crackargs = crackargs
-        self.dsthost, self.dstport = host, int(port)
+        try:
+            if int(port) > 1 and int(port) <= 65535:
+                self.dsthost, self.dstport = host, int(port)
+        except (ValueError, TypeError):
+            self.log.error('port should strictly be an integer between 1 and 65535')
+            exit(1)
         self.domain = self.dsthost
         if domain:
             self.domain = domain
@@ -110,7 +113,7 @@ class ASipOfRedWine:
         self.localport = localport
         self.requesturi = requesturi
         self.noncecount = 1
-        self.originallocalport = localport        
+        self.originallocalport = localport
         if self.sessionpath is not None:
             self.packetcount = packetcounter(50)
         if externalip is None:
@@ -179,28 +182,29 @@ class ASipOfRedWine:
     def getResponse(self):
         # we got stuff to read off the socket
         buff, _ = self.sock.recvfrom(8192)
-        buff = buff.decode('utf-8')
+        buff = buff.decode('utf-8', 'ignore')
         if buff.startswith(self.PROXYAUTHREQ):
             self.dstisproxy = True
         elif buff.startswith(self.AUTHREQ):
             self.dstisproxy = False
         if buff.startswith(self.PROXYAUTHREQ) or buff.startswith(self.AUTHREQ):
             authheader = getAuthHeader(buff)
-            nonce = getNonce(authheader)
-            opaque = getOpaque(authheader)
-            algorithm = getAlgorithm(authheader)
-            qop = getQop(authheader)
-            cid = getCID(buff)
-            if self.realm is None:
-                self.realm = getRealm(buff)
-            if None not in (nonce, self.realm):
-                if self.reusenonce:
-                    if len(self.challenges) > 0:
-                        return
-                    else:
-                        self.staticnonce = nonce
-                        self.staticcid = cid
-                self.challenges.append([nonce, cid, qop, algorithm, opaque])
+            if authheader is not None:
+                nonce = getNonce(authheader)
+                opaque = getOpaque(authheader)
+                algorithm = getAlgorithm(authheader)
+                qop = getQop(authheader)
+                cid = getCID(buff)
+                if self.realm is None:
+                    self.realm = getRealm(buff)
+                if None not in (nonce, self.realm):
+                    if self.reusenonce:
+                        if len(self.challenges) > 0:
+                            return
+                        else:
+                            self.staticnonce = nonce
+                            self.staticcid = cid
+                    self.challenges.append([nonce, cid, qop, algorithm, opaque])
         elif buff.startswith(self.OKEY):
             self.passwordcracked = True
             _tmp = getCredentials(buff)
@@ -216,7 +220,7 @@ class ASipOfRedWine:
                 self.noauth = True
                 self.resultpasswd[self.username] = '[no password]'
         elif buff.startswith(self.NOTFOUND):
-            self.log.warn("User not found")
+            self.log.warning("User not found")
             self.noauth = True
         elif buff.startswith(self.INVALIDPASS):
             pass
@@ -244,7 +248,7 @@ class ASipOfRedWine:
                 self.log.debug("could not bind to %s" % self.localport)
                 self.localport += 1
         if self.originallocalport != self.localport:
-            self.log.warn("could not bind to %s:%s - some process might already be listening on this port. Listening on port %s instead" %
+            self.log.warning("could not bind to %s:%s - some process might already be listening on this port. Listening on port %s instead" %
                           (self.bindingip, self.originallocalport, self.localport))
             self.log.info(
                 "Make use of the -P option to specify a port to bind to yourself")
@@ -282,14 +286,14 @@ class ASipOfRedWine:
                     self.getResponse()
                     self.lastrecvtime = time.time()
                 except socket.error as err:
-                    self.log.warn("socket error: %s" % err)
+                    self.log.warning("socket error: %s" % err)
             else:
                 # check if its been a while since we had a response to prevent
                 # flooding - otherwise stop
                 timediff = time.time() - self.lastrecvtime
                 if timediff > self.maxlastrecvtime:
                     self.nomore = True
-                    self.log.warn(
+                    self.log.warning(
                         'It has been %s seconds since we last received a response - stopping' % timediff)
                 if self.passwordcracked:
                     break
@@ -346,7 +350,7 @@ class ASipOfRedWine:
                                     self.log.debug(
                                         'logged last position %s' % self.crackargs.tell())
                             except IOError:
-                                self.log.warn(
+                                self.log.warning(
                                     'could not log the last extension scanned')
                 except socket.error as err:
                     self.log.error("socket error: %s" % err)
@@ -360,6 +364,8 @@ def main():
     usage += "%prog -u100 -r1-9999 -z4 10.0.0.1\r\n"
     parser = OptionParser(usage, version="%prog v" +
                           str(__version__) + __GPL__)
+    parser.add_option("-p", "--port", dest="port", default="5060",
+                      help="Destination port of the SIP device - eg -p 5060", metavar="PORT")
     parser = standardoptions(parser)
     parser = standardscanneroptions(parser)
     parser.add_option("-u", "--username", dest="username",
@@ -385,20 +391,18 @@ def main():
                       help="""A format string which allows us to specify a template for the extensions
                       example svwar.py -e 1-999 --template="123%#04i999" would scan between 1230001999 to 1230999999"
                       """)
-    parser.add_option('--maximumtime', action='store', dest='maximumtime', type="int",
-                      default=10,
-                      help="""Maximum time in seconds to keep sending requests without
-                      receiving a response back""")
+    parser.add_option('--maximumtime', action='store', dest='maximumtime', type="int", default=10,
+                      help="Maximum time in seconds to keep sending requests without receiving a response back")
     parser.add_option('--enabledefaults', '-D', action="store_true", dest="defaults",
                       default=False, help="""Scan for default / typical passwords such as
                       1000,2000,3000 ... 1100, etc. This option is off by default.
                       Use --enabledefaults to enable this functionality""")
     parser.add_option('--domain', dest="domain",
-                      help="force a specific domain name for the SIP message, eg. -d example.org")
+                      help="force a specific domain name for the SIP message, eg. example.org")
     parser.add_option('--requesturi', dest="requesturi",
                         help="force the first line URI to a specific value; e.g. sip:999@example.org")
-    parser.add_option('-6', dest="ipv6", action="store_true", help="scan an IPv6 address")
-    parser.add_option('-m','--method', dest='method', default='REGISTER')
+    parser.add_option('-6', dest="ipv6", action="store_true", help="Scan an IPv6 address")
+    parser.add_option('-m','--method', dest='method', default='REGISTER', help="Specify a SIP method to use")
     (options, args) = parser.parse_args()
     exportpath = None
     logging.basicConfig(level=calcloglevel(options))
@@ -451,7 +455,7 @@ def main():
     if options.dictionary is not None:
         crackmode = 2
         try:
-            dictionary = open(options.dictionary, 'r')
+            dictionary = open(options.dictionary, 'r', encoding='utf-8', errors='ignore')
         except IOError:
             logging.error("could not open %s" % options.dictionary)
         if options.resume is not None:
@@ -480,7 +484,7 @@ def main():
             exportpath = os.path.join(os.path.expanduser(
                 '~'), '.sipvicious', __prog__, options.save)
             if os.path.exists(exportpath):
-                logging.warn(
+                logging.warning(
                     'we found a previous scan with the same name. Please choose a new session name')
                 exit(1)
             logging.debug('creating an export location %s' % exportpath)
@@ -525,7 +529,7 @@ def main():
         if exportpath is not None:
             open(os.path.join(exportpath, 'closed'), 'w').close()
     except KeyboardInterrupt:
-        logging.warn('caught your control^c - quiting')
+        logging.warning('caught your control^c - quiting')
     except Exception as err:
         if options.reportBack:
             logging.critical(
@@ -551,7 +555,7 @@ def main():
                 logging.debug('logged last position %s' %
                               sipvicious.crackargs.tell())
         except IOError:
-            logging.warn('could not log the last tried password')
+            logging.warning('could not log the last tried password')
     # display results
     if not options.quiet:
         lenres = len(sipvicious.resultpasswd)
@@ -568,9 +572,9 @@ def main():
                         rows.append((k, sipvicious.resultpasswd[k]))
                 print(to_string(rows, header=labels))
             else:
-                logging.warn("too many to print - use svreport for this")
+                logging.warning("too many to print - use svreport for this")
         else:
-            logging.warn("found nothing")
+            logging.warning("found nothing")
     end_time = datetime.now()
     total_time = end_time - start_time
     logging.info("Total time: %s" % total_time)
