@@ -1,10 +1,9 @@
-#!/usr/bin/env python3
 # svmap.py - SIPvicious SIP scanner
 
 __GPL__ = """
 
    SIPvicious SIP scanner searches for SIP devices on a given network
-   Copyright (C) 2007-2020  Sandro Gauci <sandro@enablesecurity.com>
+   Copyright (C) 2007-2021 Sandro Gauci <sandro@enablesecurity.com>
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -28,34 +27,36 @@ import random
 import select
 import socket
 import traceback
+from struct import pack
+from sys import exit
 from datetime import datetime
 from optparse import OptionParser
-from struct import pack, unpack
-from sys import exit
-
-from .libs.pptable import to_string
-from .libs.svhelper import (
-    __version__, calcloglevel, createTag, fingerPrintPacket, getranges,  
-    getTag, getTargetFromSRV, ip4range, makeRequest, getRange, scanlist,
-    mysendto, packetcounter, reportBugToAuthor, dbexists, scanfromfile, 
+from sipvicious.libs.pptable import to_string
+from sipvicious.libs.svhelper import (
+    __version__, calcloglevel, createTag, fingerPrintPacket, getranges,
+    getTag, getTargetFromSRV, ip4range, makeRequest, getRange, scanlist, ip6range,
+    mysendto, packetcounter, reportBugToAuthor, dbexists, scanfromfile, check_ipv6,
     scanrandom, standardoptions, standardscanneroptions, resumeFromIP, scanfromdb
 )
 
 __prog__ = "svmap"
 
 class DrinkOrSip:
-    def __init__(self,scaniter,selecttime=0.005,compact=False, bindingip='0.0.0.0',
+    def __init__(self,scaniter,selecttime=0.005,compact=False, bindingip='',
                  fromname='sipvicious',fromaddr='sip:100@1.1.1.1', extension=None,
                  sessionpath=None,socktimeout=3,externalip=None,localport=5060,
-                 printdebug=False,first=None,fpworks=False):
+                 printdebug=False,first=None,fpworks=False,ipv6=False):
         self.log = logging.getLogger('DrinkOrSip')
+        family = socket.AF_INET
+        if ipv6:
+            family = socket.AF_INET6
+        self.ipv6 = ipv6
         self.bindingip = bindingip
         self.sessionpath = sessionpath
         self.dbsyncs = False
         if self.sessionpath is not  None:
             self.resultip = dbm.open(os.path.join(self.sessionpath,'resultip'),'c')
             self.resultua = dbm.open(os.path.join(self.sessionpath,'resultua'),'c')
-            self.resultfp = dbm.open(os.path.join(self.sessionpath,'resultfp'),'c')
             try:
                 self.resultip.sync()
                 self.dbsyncs = True
@@ -66,9 +67,8 @@ class DrinkOrSip:
         else:
             self.resultip = dict()
             self.resultua = dict()
-            self.resultfp = dict()
         # we do UDP
-        self.sock = socket.socket(socket.AF_INET,socket.SOCK_DGRAM)
+        self.sock = socket.socket(family, socket.SOCK_DGRAM)
         # socket timeout - this is particularly useful when quitting .. to eat
         # up some final packets
         self.sock.settimeout(socktimeout)
@@ -114,17 +114,17 @@ class DrinkOrSip:
             self.packetcount = packetcounter(50)
         self.sentpackets = 0
 
-    def getResponse(self,buff,srcaddr):        
-        srcip,srcport = srcaddr
+    def getResponse(self,buff,srcaddr):
+        srcip, srcport, *_ = srcaddr
         uaname = 'unknown'
-        buff = buff.decode('utf-8')
+        buff = buff.decode('utf-8', 'ignore')
         if buff.startswith('OPTIONS ') \
             or buff.startswith('INVITE ') \
             or buff.startswith('REGISTER '):
             if self.externalip == srcip:
-                self.log.debug("We received our own packet from %s:%s" % srcaddr)
+                self.log.debug("We received our own packet from %s:%s" % (str(srcip), srcport))
             else:
-                self.log.info("Looks like we received a SIP request from %s:%s"% srcaddr)
+                self.log.info("Looks like we received a SIP request from %s:%s"% (str(srcip), srcport))
                 self.log.debug(buff.__repr__())
             return
         self.log.debug("running fingerPrintPacket()")
@@ -154,15 +154,13 @@ class DrinkOrSip:
             except (ValueError,TypeError,socket.error):
                 self.log.debug("original destination could not be decoded: %s" % (originaldst))
                 dstip,dstport = 'unknown','unknown'
-            resultstr = '%s:%s\t->\t%s:%s\t->\t%s\t->\t%s' % (dstip,dstport,srcip,srcport,uaname,fpname)
+            resultstr = '%s:%s\t->\t%s:%s\t->\t%s' % (dstip,dstport,srcip,srcport,uaname)
             self.log.info( resultstr )
             self.resultip['%s:%s' % (srcip,srcport)] = '%s:%s' % (dstip,dstport)
             self.resultua['%s:%s' % (srcip,srcport)] = uaname
-            self.resultfp['%s:%s' % (srcip,srcport)] = fpname
             if self.sessionpath is not None and self.dbsyncs:
-                    self.resultip.sync()
-                    self.resultua.sync()
-                    self.resultfp.sync()
+                self.resultip.sync()
+                self.resultua.sync()
         else:
             self.log.info('Packet from %s:%s did not contain a SIP msg'%srcaddr)
             self.log.debug('Packet: %s' % buff.__repr__())
@@ -171,7 +169,11 @@ class DrinkOrSip:
         # bind to 5060 - the reason is to maximize compatability with
         # devices that disregard the source port and send replies back
         # to port 5060
-        self.log.debug("binding to %s:%s" % (self.bindingip,self.localport))
+        if self.bindingip == '':
+            bindingip = 'any'
+        else:
+            bindingip = self.bindingip
+        self.log.debug("binding to %s:%s" % (bindingip, self.localport))
         while 1:
             if self.localport > 65535:
                 self.log.critical("Could not bind to any port")
@@ -183,7 +185,7 @@ class DrinkOrSip:
                 self.log.debug("could not bind to %s" % self.localport)
                 self.localport += 1
         if self.originallocalport != self.localport:
-            self.log.warn("could not bind to %s:%s - some process might already be listening on this port. Listening on port %s instead" % (self.bindingip,self.originallocalport, self.localport))
+            self.log.warning("could not bind to %s:%s - some process might already be listening on this port. Listening on port %s instead" % (self.bindingip,self.originallocalport, self.localport))
             self.log.info("Make use of the -P option to specify a port to bind to yourself")
         while 1:
             r, _, _ = select.select(
@@ -196,7 +198,8 @@ class DrinkOrSip:
                 # we got stuff to read off the socket
                 try:
                     buff,srcaddr = self.sock.recvfrom(8192)
-                    self.log.debug('got data from %s:%s' % srcaddr)
+                    host, port, *_ = srcaddr
+                    self.log.debug('got data from %s:%s' % (str(host), str(port)))
                     self.log.debug('data: %s' % buff.__repr__())
                     if self.printdebug:
                         print(srcaddr)
@@ -228,10 +231,19 @@ class DrinkOrSip:
                 dstip,dstport,method = nextscan
                 self.nextip = dstip
                 dsthost = (dstip,dstport)
+                domain = dsthost[0]
                 branchunique = '%s' % random.getrandbits(32)
-
-                localtag = createTag('%s%s' % (''.join(map(lambda x: '%02x' % int(x), dsthost[0].split('.'))),'%04x' % dsthost[1]))
-                fromaddr = '"%s"<%s>' % (self.fromname,self.fromaddr)
+                if self.ipv6 and check_ipv6(dsthost[0]):
+                    domain = '[' + dsthost[0] + ']'
+                    localtag = createTag('%s%s' % (''.join(map(lambda x:
+                        '%s' % x, dsthost[0].split(':'))), '%04x' % dsthost[1]))
+                else:
+                    localtag = createTag('%s%s' % (''.join(map(lambda x:
+                        '%02x' % int(x), dsthost[0].split('.'))),'%04x' % dsthost[1]))
+                if self.ipv6:
+                    fromaddr = '"%s"<sip:100@%s>' % (self.fromname, domain)
+                else:
+                    fromaddr = '"%s"<%s>' % (self.fromname, self.fromaddr)
                 toaddr = fromaddr
                 callid = '%s' % random.getrandbits(80)
                 contact = None
@@ -241,7 +253,7 @@ class DrinkOrSip:
                                 method,
                                 fromaddr,
                                 toaddr,
-                                dsthost[0],
+                                domain,
                                 dsthost[1],
                                 callid,
                                 self.externalip,
@@ -267,7 +279,7 @@ class DrinkOrSip:
                                 f.close()
                                 self.log.debug('logged last ip %s' % self.nextip)
                             except IOError:
-                                self.log.warn('could not log the last ip scanned')
+                                self.log.warning('could not log the last ip scanned')
                     if self.first is not None:
                         if self.sentpackets >= self.first:
                             self.log.info('Reached the limit to scan the first %s packets' % self.first)
@@ -287,6 +299,8 @@ def main():
     usage += "%prog --resume session1 -v\r\n\r\n"
     usage += "%prog -p5060-5062 10.0.0.3-20 -m INVITE\r\n\r\n"
     parser = OptionParser(usage, version="%prog v"+str(__version__)+__GPL__)
+    parser.add_option("-p", "--port", dest="port", default="5060",
+                      help="Destination port or port ranges of the SIP device - eg -p5060,5061,8000-8100", metavar="PORT")
     parser = standardoptions(parser)
     parser = standardscanneroptions(parser)
     parser.add_option("--randomscan", dest="randomscan", action="store_true",
@@ -296,30 +310,23 @@ def main():
                   help="Scan IPs which were found in a previous scan. Pass the session name as the argument", metavar="scan1")
     parser.add_option("-I", "--inputtext", dest="inputtext",
                   help="Scan IPs from a text file - use the same syntax as command line but with new lines instead of commas. Pass the file name as the argument", metavar="scan1")
-    parser.add_option("-m", "--method", dest="method",
-                  help="Specify the request method - by default this is OPTIONS.",
-                  default='OPTIONS'
-                  )
+    parser.add_option("-m", "--method", dest="method",  help="Specify the request method - by default this is OPTIONS.",
+                  default='OPTIONS')
     parser.add_option("-d", "--debug", dest="printdebug",
-                  help="Print SIP messages received",
-                  default=False, action="store_true"
-                  )
+                  help="Print SIP messages received", default=False, action="store_true")
     parser.add_option("--first", dest="first",
                   help="Only send the first given number of messages (i.e. usually used to scan only X IPs)",
-                  type="long",
-                  )
+                  type="long")
     parser.add_option("-e", "--extension", dest="extension", default='100',
                   help="Specify an extension - by default this is not set")
-    parser.add_option("--randomize", dest="randomize", action="store_true",
-                      default=False,
+    parser.add_option("--randomize", dest="randomize", action="store_true", default=False,
                   help="Randomize scanning instead of scanning consecutive ip addresses")
-    parser.add_option("--srv", dest="srvscan", action="store_true",
-                      default=False,
+    parser.add_option("--srv", dest="srvscan", action="store_true", default=False,
                   help="Scan the SRV records for SIP on the destination domain name." \
                        "The targets have to be domain names - example.org domain1.com")
     parser.add_option('--fromname',dest="fromname", default="sipvicious",
                       help="specify a name for the from header")
-    parser.add_option('--crashandburn', dest="crashandburn", action="store_true", default=False)
+    parser.add_option('-6', '--ipv6', dest="ipv6", action='store_true', help="scan an IPv6 address")
     (options, args) = parser.parse_args()
     exportpath = None
     if options.resume is not None:
@@ -434,19 +441,21 @@ def main():
                 args = resumeFromIP(previousip,args)
                 logging.debug('New args: %s' % args)
                 logging.info('Resuming from %s' % previousip)
-
-            # normal consecutive scan
-            try:
-                iprange = ip4range(*args)
-            except ValueError as err:
-                logging.error(err)
-                exit(1)
-            scaniter = scanlist(iprange,portrange,options.method.split(','))
+            if options.ipv6:
+                scaniter = scanlist(ip6range(*args), portrange, options.method.split(','))
+            else:
+                # normal consecutive scan
+                try:
+                    iprange = ip4range(*args)
+                except ValueError as err:
+                    logging.error(err)
+                    exit(1)
+                scaniter = scanlist(iprange,portrange,options.method.split(','))
     if options.save is not None:
         if options.resume is None:
             exportpath = os.path.join(os.path.expanduser('~'),'.sipvicious',__prog__,options.save)
             if os.path.exists(exportpath):
-                logging.warn('we found a previous scan with the same name. Please choose a new session name')
+                logging.warning('we found a previous scan with the same name. Please choose a new session name')
                 exit(1)
             logging.debug('creating an export location %s' % exportpath)
             try:
@@ -478,12 +487,11 @@ def main():
                     printdebug=options.printdebug,
                     first=options.first,
                     fromname=options.fromname,
+                    ipv6=options.ipv6,
                     )
     start_time = datetime.now()
     logging.info( "start your engines" )
     try:
-        if options.crashandburn:
-            raise ValueError
         try:
             sipvicious.start()
         except AssertionError as err:
@@ -492,7 +500,7 @@ def main():
         if exportpath is not None:
             open(os.path.join(exportpath,'closed'),'w').close()
     except KeyboardInterrupt:
-        logging.warn( 'caught your control^c - quiting' )
+        logging.warning( 'caught your control^c - quiting' )
         pass
     except Exception as err:
         if options.reportBack:
@@ -510,7 +518,7 @@ def main():
             pickle.dump(sipvicious.nextip,f)
             f.close()
         except OSError:
-            logging.warn('Could not save state to %s' % lastipdst)
+            logging.warning('Could not save state to %s' % lastipdst)
     elif options.save is None:
         if scanrandomstore is not None:
         #if options.randomize or options.randomscan:
@@ -518,7 +526,7 @@ def main():
                     logging.debug('removing %s' % scanrandomstore)
                     os.unlink(scanrandomstore)
             except OSError:
-                    logging.warn('could not remove %s' % scanrandomstore)
+                    logging.warning('could not remove %s' % scanrandomstore)
                     pass
     # display results
     if not options.quiet:
@@ -527,19 +535,19 @@ def main():
             logging.info("we have %s devices" % lenres)
             if (lenres < 400 and options.save is not None) or options.save is None:
                 width = 60
-                labels = ('SIP Device','User Agent','Fingerprint')
+                labels = ('SIP Device','User Agent')
                 rows = list()
                 try:
                     for k in sipvicious.resultua.keys():
-                        rows.append((k.decode(),sipvicious.resultua[k].decode(),sipvicious.resultfp[k].decode()))
+                        rows.append((k.decode(),sipvicious.resultua[k].decode()))
                 except AttributeError:
                     for k in sipvicious.resultua.keys():
-                        rows.append((k,sipvicious.resultua[k],sipvicious.resultfp[k]))  
+                        rows.append((k,sipvicious.resultua[k]))
                 print(to_string(rows, header=labels))
             else:
-                logging.warn("too many to print - use svreport for this")
+                logging.warning("too many to print - use svreport for this")
         else:
-            logging.warn("found nothing")
+            logging.warning("found nothing")
     end_time = datetime.now()
     total_time = end_time - start_time
     logging.info("Total time: %s" %  total_time)
