@@ -1,10 +1,9 @@
-#!/usr/bin/env python3
 # svwar.py - SIPvicious extension line scanner
 
 __GPL__ = """
 
    Sipvicious extension line scanner scans SIP PaBXs for valid extension lines
-   Copyright (C) 2007-2020 Sandro Gauci <sandro@enablesecurity.com>
+   Copyright (C) 2007-2021 Sandro Gauci <sandro@enablesecurity.com>
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -28,17 +27,14 @@ import socket
 import time
 import dbm
 import os
-import re
 import traceback
 from sys import exit
 from optparse import OptionParser
 from datetime import datetime
-from socket import error as socketerror
-from base64 import b64decode, b64encode
-from .libs.pptable import to_string
-from .libs.svhelper import (
+from sipvicious.libs.pptable import to_string
+from sipvicious.libs.svhelper import (
     __version__, numericbrute, dictionaryattack, mysendto,
-    createTag, check_ipv6, makeRequest, getTag, parseHeader, 
+    createTag, check_ipv6, makeRequest, getTag, parseHeader,
     getRealm, standardoptions, standardscanneroptions, calcloglevel,
     resumeFrom, getRange, reportBugToAuthor, packetcounter
 )
@@ -83,7 +79,12 @@ class TakeASip:
         self.xlist = list()
         self.challenges = list()
         self.realm = None
-        self.dsthost, self.dstport = host, int(port)
+        try:
+            if int(port) > 1 and int(port) <= 65535:
+                self.dsthost, self.dstport = host, int(port)
+        except (ValueError, TypeError):
+            self.log.error('port should strictly be an integer between 1 and 65535')
+            exit(1)
         self.domain = self.dsthost
         if domain:
             self.domain = domain
@@ -99,7 +100,7 @@ class TakeASip:
         self.BADUSER = None
         self.method = method.upper()
         if self.method == 'INVITE':
-            self.log.warn(
+            self.log.warning(
                 'using an INVITE scan on an endpoint (i.e. SIP phone) may cause it to ring and wake up people in the middle of the night')
         if self.sessionpath is not None:
             self.packetcount = packetcounter(50)
@@ -150,7 +151,7 @@ class TakeASip:
     # try with the next one.
     SERVICEUN = 'SIP/2.0 503 '
 
-    def createRequest(self, m, username=None, auth=None, cid=None, 
+    def createRequest(self, m, username=None, auth=None, cid=None,
                         cseq=1, fromaddr=None, toaddr=None, contact=None):
         if cid is None:
             cid = '%s' % str(random.getrandbits(32))
@@ -192,7 +193,7 @@ class TakeASip:
             print(buff)
         buff = buff.decode('utf-8')
         try:
-            extension = getTag(buff).decode('utf-8')
+            extension = getTag(buff).decode('utf-8', 'ignore')
         except (TypeError, AttributeError):
             self.log.error('could not decode to tag')
             extension = None
@@ -227,7 +228,7 @@ class TakeASip:
                     # _tmp['headers']['from'][0].split('"')[1]
                     username = getTag(buff)
                 except IndexError:
-                    self.log.warn('could not parse the from address %s' % _tmp[
+                    self.log.warning('could not parse the from address %s' % _tmp[
                                   'headers']['from'])
                     username = 'XXX'
                 cseq = _tmp['headers']['cseq'][0]
@@ -280,7 +281,7 @@ class TakeASip:
                 if self.sessionpath is not None and self.dbsyncs:
                     self.resultauth.sync()
             else:
-                self.log.warn(
+                self.log.warning(
                     "extension '%s' probably exists but the response is unexpected" % extension)
                 self.log.debug("response: %s" % firstline)
                 self.resultauth[extension] = 'weird'
@@ -304,7 +305,7 @@ class TakeASip:
         elif buff.startswith(self.DECLINED):
             pass
         elif buff.startswith(self.NOTALLOWED):
-            self.log.warn("method not allowed")
+            self.log.warning("method not allowed")
             self.nomore = True
             return
         elif buff.startswith(self.BADREQUEST):
@@ -313,7 +314,7 @@ class TakeASip:
             self.nomore = True
             return
         else:
-            self.log.warn("We got an unknown response")
+            self.log.warning("We got an unknown response")
             self.log.error("Response: %s" % buff.__repr__())
             self.log.debug("1st line: %s" % firstline.__repr__())
             self.log.debug("Bad user: %s" % self.BADUSER.__repr__())
@@ -336,7 +337,7 @@ class TakeASip:
                 self.log.debug("could not bind to %s" % self.localport)
                 self.localport += 1
         if self.originallocalport != self.localport:
-            self.log.warn("could not bind to %s:%s - some process might already be listening on this port. Listening on port %s instead" %
+            self.log.warning("could not bind to %s:%s - some process might already be listening on this port. Listening on port %s instead" %
                           (self.bindingip, self.originallocalport, self.localport))
             self.log.info(
                 "Make use of the -P option to specify a port to bind to yourself")
@@ -363,7 +364,7 @@ class TakeASip:
                 except socket.error as err:
                     self.log.error("socket error: %s" % err)
                     return
-                buff = buff.decode('utf-8')
+                buff = buff.decode('utf-8', 'ignore')
                 if buff.startswith(self.TRYING) \
                         or buff.startswith(self.RINGING) \
                         or buff.startswith(self.UNAVAILABLE):
@@ -394,7 +395,7 @@ class TakeASip:
             self.log.error("socket error: %s" % err)
             return
         if self.BADUSER.startswith(self.AUTHREQ):
-            self.log.warn(
+            self.log.warning(
                 "Bad user = %s - svwar will probably not work!" % self.AUTHREQ)
         # let the fun commence
         self.log.info('Ok SIP device found')
@@ -421,7 +422,7 @@ class TakeASip:
                 timediff = time.time() - self.lastrecvtime
                 if timediff > self.maxlastrecvtime:
                     self.nomore = True
-                    self.log.warn(
+                    self.log.warning(
                         'It has been %s seconds since we last received a response - stopping' % timediff)
                     continue
                 # no stuff to read .. its our turn to send back something
@@ -452,7 +453,7 @@ class TakeASip:
                                     self.log.debug(
                                         'logged last position %s' % self.guessargs.tell())
                             except IOError:
-                                self.log.warn(
+                                self.log.warning(
                                     'could not log the last extension scanned')
                 except socket.error as err:
                     self.log.error("socket error: %s" % err)
@@ -466,6 +467,8 @@ def main():
     usage += "%prog -d dictionary.txt 10.0.0.2\r\n"
     parser = OptionParser(usage, version="%prog v" +
                           str(__version__) + __GPL__)
+    parser.add_option("-p", "--port", dest="port", default="5060",
+                      help="Destination port of the SIP device - eg -p 5060", metavar="PORT")
     parser = standardoptions(parser)
     parser = standardscanneroptions(parser)
     parser.add_option("-d", "--dictionary", dest="dictionary", type="string",
@@ -547,7 +550,7 @@ def main():
     if options.dictionary is not None:
         guessmode = 2
         try:
-            dictionary = open(options.dictionary, 'r')
+            dictionary = open(options.dictionary, 'r', encoding='utf-8', errors='ignore')
         except IOError:
             logging.error("could not open %s" % options.dictionary)
             exit(1)
@@ -577,7 +580,7 @@ def main():
             exportpath = os.path.join(os.path.expanduser(
                 '~'), '.sipvicious', __prog__, options.save)
             if os.path.exists(exportpath):
-                logging.warn(
+                logging.warning(
                     'we found a previous scan with the same name. Please choose a new session name')
                 exit(1)
             logging.debug('creating an export location %s' % exportpath)
@@ -624,7 +627,7 @@ def main():
         if exportpath is not None:
             open(os.path.join(exportpath, 'closed'), 'w').close()
     except KeyboardInterrupt:
-        logging.warn('caught your control^c - quiting')
+        logging.warning('caught your control^c - quiting')
     except Exception as err:
         if options.reportBack:
             logging.critical(
@@ -649,7 +652,7 @@ def main():
                 logging.debug('logged last position %s' %
                               sipvicious.guessargs.tell())
         except IOError:
-            logging.warn('could not log the last extension scanned')
+            logging.warning('could not log the last extension scanned')
     # display results
     if not options.quiet:
         lenres = len(sipvicious.resultauth)
@@ -663,12 +666,12 @@ def main():
                         rows.append((k.decode(), sipvicious.resultauth[k].decode()))
                 except AttributeError:
                     for k in sipvicious.resultauth.keys():
-                        rows.append((k, sipvicious.resultauth[k]))                    
+                        rows.append((k, sipvicious.resultauth[k]))
                 print(to_string(rows, header=labels))
             else:
-                logging.warn("too many to print - use svreport for this")
+                logging.warning("too many to print - use svreport for this")
         else:
-            logging.warn("found nothing")
+            logging.warning("found nothing")
     end_time = datetime.now()
     total_time = end_time - start_time
     logging.info("Total time: %s" % total_time)
